# Schedule.pm
#	Copyright(C)2001 Ktat, All rights reserved.

package Schedule;

use strict;
use vars qw($db $sp_db $db_file $sp_db_file);
use Calendar;
require 'jcode.pl';

sub new{
	my $pkg  = shift;
	my $dir  = shift;
	my $year = shift;
	my $month= shift;
	my $admin= shift;
	my $message = '';
	$db_file    = $dir."/".$year.$month.".sch";
	$sp_db_file = $dir."/sp_db.sch";
	($db,$sp_db,$message) = _read_db();
	bless{
		'dir' => $dir,
		'mode'=> $admin,
		'message' => $message
	},$pkg;
}

sub _read_db{
	my %db;
	my %sp_db;
	my $message = '';
	if(_f_lock("check")){
		if(open DB,$db_file){
			seek DB,0,0;
			while(<DB>){
				chomp;
				my($date,@data) = split /\t/;
				$db{$date} = \@data;
			}
		  close DB;
		}
		if(open DB,$sp_db_file){
			seek DB,0,0;
			while(<DB>){
				chomp;
				my($date,@data) = split /\t/;
				$sp_db{$date} = \@data;
			}
		  close DB;
		}
	}else{
		$message = "BUSY (Schedule::_read_db)";
	}
	return(\%db,\%sp_db,$message);
}

sub rec_db{
	my $self = shift;
	my $judge= 1;
	if(_f_lock("on")){
		if(open DB,">$db_file"){
			seek DB,0,0;
			foreach my $date (keys %{$db}){
				print DB $date."\t".join("\t",@{$db->{$date}})."\n";
			}
		 close DB;
		}else{
			$self->{message} = "Can't make a new file (Schedule::rec_db)";
			$judge = 0;
		}
	_f_lock();
	}else{
		$self->{message} = "Lock was failed (Schedule::rec_db)";
		$judge = 0;
	}
	return $judge;
}

sub rec_sp_db{
	my $self = shift;
	my $judge= 1;
	if(_f_lock("on")){
		if(open DB,">$sp_db_file"){
			seek DB,0,0;
			foreach my $date (keys %{$sp_db}){
				print DB $date."\t".join("\t",@{$sp_db->{$date}})."\n";
			}
		 close DB;
		}else{
			$self->{message} = "Can't make a new file (Schedule::rec_sp_db2)";
			$judge = 0;
		}
	_f_lock();
	}else{
		$self->{message} = "Lock was failed (Schedule::rec_sp_db2)";
		$judge = 0;
	}
	return $judge;
}

sub get_sp_reserved{
	my $self = shift;
	my $date = shift;
	my $reserved = [()];
	my $month = substr($date, 4,2) + 0;
	my $day   = substr($date,-2) + 0;
	my $week = $day >= 29 ? 4 : int(($day - 1) / 7);
	my $cal  = new Calendar (substr($date,0,4),$month);
	my $_day = $cal->calc($day);
	push(@{$reserved},@{$sp_db->{"1-$month-$week-$_day"}})	if exists($sp_db->{"1-$month-$week-$_day"});
	push(@{$reserved},@{$sp_db->{"1-*-$week-$_day"}})		if exists($sp_db->{"1-*-$week-$_day"});
	push(@{$reserved},@{$sp_db->{"1-$month-*-$_day"}})		if exists($sp_db->{"1-$month-*-$_day"});
	push(@{$reserved},@{$sp_db->{"1-*-*-$_day"}})			if exists($sp_db->{"1-*-*-$_day"});
	push(@{$reserved},@{$sp_db->{"2-$month-$day"}})			if exists($sp_db->{"2-$month-$day"});
	push(@{$reserved},@{$sp_db->{"2-*-$day"}})				if exists($sp_db->{"2-*-$day"});
	return $reserved;
}

sub get_reserved{
	my $self = shift;
	my $date = shift;
	return exists($db->{$date}) ? $db->{$date} : [()];
}

sub reserve{
	my $self = shift;
	my %RS = %{shift()};
	my $date = $RS{'Date'};
	my $name = $RS{'name'};
	my $mail = $RS{'mail'};
	my $pass = $RS{'pass'};
	my $subject = $RS{'subject'};
	my $content = $RS{'content'};
	my $event = $RS{event};
	my $w_time  = time;
	if(	$name ne "" and
#		$mail ne "" and
		$content ne "" and
		$pass ne "" and
		($self->{'mode'} == 0 or $self->_check_pass($pass) == 1)
	){
		my $judge;
		$pass = do_crypt($pass);
		_decode(\$content);
		_decode(\$name);
		_decode(\$mail);
		_decode(\$subject);
		$content=~s/<br>//g	if $RS{br} eq "on";
		$content =~ s/(http:\/\/[A-Za-z0-9\$\#\~\.\/\-\?\=\_\%\&]+)/<a href\=\"$1\" target\=\"\_blank\">$1<\/a>/g;
		
		push(@{$db->{$date}},"$name<>$mail<>$pass<>$event>$subject<>$content<>$w_time<>$ENV{REMOTE_ADDR}");
		if( @{$db->{$date}} > 1 and 
			(split(/<>/,$db->{$date}->[$#{$db->{$date}}]))[4] eq (split(/<>/,$db->{$date}->[$#{$db->{$date}}-1]))[4]
		){
			pop(@{$db->{$date}});
			$self->{message} = "ݏo܂łB<br>\n";
			$self->{message} .= "ȅ݂łB<br>\n";
			return 0;
		}else{
			return 1;
		}
	}else{
		$self->{message} = "ݏo܂łB<br>\n";
		$self->{message} .= "ėpɃ`FbNȂAełB<br>\n"	if $content eq "";
		$self->{message} .= "OłB<br>\n"							if $name eq "";
		$self->{message} .= "pX[hłB<br>\n" 					if $pass eq "";
		return 0;
	}
}

sub replace{
	my $self = shift;
	my %RS = %{shift()};
	my $date = $RS{Date};
	my $name = $RS{name};
	my $mail = $RS{mail};
	my $pass = $RS{pass};
	my $event = $RS{event};
	my $subject = $RS{'subject'};
	my $content = $RS{'content'};
	my $del_num = $RS{del_num};
	my $w_time  = time;
	my $judge = 0;
	my $i = 0;
	foreach(@{$db->{$date}}){
		$i++;
		my @param = split(/<>/);
		if(
	#		$param[0] eq $RS{mail} and
			$del_num == $i and
			$param[2] eq crypt($pass,$param[2])
		){
			$pass = do_crypt($pass);
			_decode(\$content);
			_decode(\$name);
			_decode(\$mail);
			_decode(\$subject);
			$content=~s/<br>//g	if $RS{br} eq "on";
			$content =~ s/(http:\/\/[A-Za-z0-9\$\#\~\.\/\-\?\=\_\%\&]+)/<a href\=\"$1\" target\=\"\_blank\">$1<\/a>/g;
			$_ = "$name<>$mail<>$pass<>$event>$subject<>$content<>$w_time<>$ENV{REMOTE_ADDR}";
			$judge = 1;
		}
	}
	$self->{message} = "uo܂łB<br>\n"	if $judge ==0;
	return $judge;
}

sub cancel{
	my $self = shift;
	my %RS = %{shift()};
	my $date = $RS{Date};
	my $mail = $RS{mail};
	my $pass = $RS{pass};
	my $del_num = $RS{del_num};
	my $judge = 0;
	my @new = ();
	my $i = 0;
	foreach(@{$db->{$date}}){
		$i++;
		my @param = split(/<>/);
		if(
	#		$param[0] eq $RS{mail} and
			$del_num == $i and
			( $param[2] eq crypt($pass,$param[2]) or 
			  $self->_check_pass($pass) == 1)
		){
			$judge = 1;
		}else{
			push(@new,$_);
		}
	}
	$judge == 1 ? @{$db->{$date}} = @new : "폜o܂łB<br>\n";
	return $judge;
}

sub sp_reserve{
	my $self = shift;
	my %RS = %{shift()};
	my $date;
	my $hanyo = $RS{hanyo};
	my $content = $RS{'content'};
	if($RS{Month} ne "" and $RS{Week} ne ""){
		$date = "1-".$RS{Month}."-".$RS{Week}."-".$RS{Day};
	}elsif($RS{_Month} ne "" and $RS{_Day} ne ""){
		$date = "2-".$RS{_Month}."-".$RS{_Day};
	}else{
		$content= "";$hanyo = 0
	}
	my $name = $RS{'name'};
	my $mail = $RS{'mail'};
	my $pass = $RS{'pass'};
	my $subject = $RS{'subject'};
	my $event = $RS{event};
	my $w_time  = time;
	if(	$name ne "" and
#		$mail ne "" and
		($hanyo == 1 or $content ne "") and
		$pass ne "" and
		($self->{'mode'} == 0 or $self->_check_pass($pass) == 1)
	){
		my $judge;
		$pass = do_crypt($pass);
		_decode(\$content);
		_decode(\$name);
		_decode(\$mail);
		_decode(\$subject);
		$content=~s/<br>//g	if $RS{br} eq "on";
		$content =~ s/(http:\/\/[A-Za-z0-9\$\#\~\.\/\-\?\=\_\%\&]+)/<a href\=\"$1\" target\=\"\_blank\">$1<\/a>/g;
		push(@{$sp_db->{$date}},"$name<>$mail<>$pass<>$hanyo>$event>$subject<>$content<>$w_time<>$ENV{REMOTE_ADDR}");
		if( @{$sp_db->{$date}} > 1 and
			(
				($hanyo !=1 and (split(/<>/,$sp_db->{$date}->[$#{$sp_db->{$date}}]))[4] eq (split(/<>/,$sp_db->{$date}->[$#{$sp_db->{$date}}-1]))[4]) or
				($hanyo ==1 and (split(/<>/,$sp_db->{$date}->[$#{$sp_db->{$date}}]))[3] eq (split(/<>/,$sp_db->{$date}->[$#{$sp_db->{$date}}-1]))[3]) 
			)
		){
			pop(@{$sp_db->{$date}});
			$self->{message} = "ݏo܂łB<br>\n";
			$self->{message} .= "ȅ݂łB<br>\n"		if$hanyo !=1;
			$self->{message} .= "薼݂̏łB<br>\n"		if$hanyo ==1;
			return 0;
		}else{
			return 1;
		}
	}else{
		$self->{message} = "ݏo܂łB<br>\n";
		$self->{message} .= "ėpɃ`FbNȂAełB<br>\n"	if $hanyo != 1 or $content eq "";
		$self->{message} .= "OłB<br>\n"							if $name eq "";
		$self->{message} .= "pX[hłB<br>\n" 					if $pass eq "";
		return 0;
	}
}

sub sp_replace{
	my $self = shift;
	my %RS = %{shift()};
	my $date;
	my $hanyo = $RS{hanyo};
	my $content = $RS{'content'};
	if($RS{Month} ne "" and $RS{Week} ne ""){
		$date = "1-".$RS{Month}."-".$RS{Week}."-".$RS{Day};
	}elsif($RS{_Month} ne "" and $RS{_Day} ne ""){
		$date = "2-".$RS{_Month}."-".$RS{_Day};
	}else{
		$content= "";$hanyo = 0
	}
	my $name = $RS{name};
	my $mail = $RS{mail};
	my $pass = $RS{pass};
	my $event = $RS{event};
	my $subject = $RS{'subject'};
	my $del_num = $RS{del_num};
	my $w_time  = time;
	my $judge = 0;
	my $i = 0;
	foreach(@{$sp_db->{$date}}){
		$i++;
		my @param = split(/<>/);
		if(
	#		$param[0] eq $RS{mail} and
			$del_num == $i and
			$param[2] eq crypt($pass,$param[2])
		){
			$pass = do_crypt($pass);
			_decode(\$content);
			_decode(\$name);
			_decode(\$mail);
			_decode(\$subject);
			$content=~s/<br>//g	if $RS{br} eq "on";
			$content =~ s/(http:\/\/[A-Za-z0-9\$\#\~\.\/\-\?\=\_\%\&]+)/<a href\=\"$1\" target\=\"\_blank\">$1<\/a>/g;
			$_ = "$name<>$mail<>$pass<>$hanyo>$event>$subject<>$content<>$w_time<>$ENV{REMOTE_ADDR}";
			$judge = 1;
		}
	}
	$self->{message} = "uo܂łB<br>\n"	if $judge ==0;
	return $judge;
}

sub sp_cancel{
	my $self = shift;
	my %RS = %{shift()};
	my $date;
	if($RS{Month} ne "" and $RS{Week} ne ""){
		$date = "1-".$RS{Month}."-".$RS{Week}."-".$RS{Day};
	}elsif($RS{_Month} ne "" and $RS{_Day} ne ""){
		$date = "2-".$RS{_Month}."-".$RS{_Day};
	}
	my $mail = $RS{mail};
	my $pass = $RS{pass};
	my $del_num = $RS{del_num};
	my $judge = 0;
	my @new = ();
	my $i = 0;
	if(substr($date,0,1) eq "0"){
		if(exists $sp_db->{$date}){
			@new = ();
			$judge = 1;
		}
	}else{
	foreach(@{$sp_db->{$date}}){
		$i++;
		my @param = split(/<>/);
		if(
	#		$param[0] eq $RS{mail} and
			$del_num == $i and
			( $param[2] eq crypt($pass,$param[2]) or 
			  $self->_check_pass($pass) == 1)
		){
			$judge = 1;
		}else{
			push(@new,$_);
		}
	}
	}
	$judge == 1 ? @{$sp_db->{$date}} = @new : "폜o܂łB<br>\n";
	return $judge;
}

sub do_crypt{
	my $pass = shift;
	my $salt = ("A".."Z","a".."z",0..9,'.','/')[rand(64)].("A".."Z","a".."z",0..9,'.','/')[rand(64)];
	return crypt($pass,$salt);
}

sub _check_pass{
	my $self = shift;
	my $pass = shift;
	if(open IN, $self->{dir}."/password"){
		seek IN,0,0;
		my $hpass =<IN>;
		close IN;
		return $hpass eq crypt($pass,$hpass) ? 1 : 0;
	}
	return 0;
}

sub _decode{
	my $value = shift;
	$$value =~s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
	jcode::convert($value,'sjis');
	$$value =~ s/\t/ /g;
	$$value=~s/[\r\n]/\n/g;
	$$value =~s/</&lt;/g;
	$$value =~s/>/&gt;/g;
	$$value =~s/\n+/<br>/g;
}

sub _f_lock{ # f_lock("on" or "" or "check")
	# _f_lock("check"):
	#	if the file isn't locked, return 1;
	# 	if the file is locked and is still locked after a few seconds return 0;
	# _f_lock("on"):
	#	if locking the file lock is success , return 1.
	# _f_lock(""):
	#	if unlocking the file is success , return 1.
	my $mode = shift;
	my $w_times = 10;
	my $success = 0;
	$db_file =~m{^(.*)/};
	my $dir = $1;
	my $lock_on = $dir."/lock";
	my $lock_off= $dir."/unlock";
	if($mode eq "check" and -e $lock_off){
		$success = 1;
	}elsif(($mode eq "on" or $mode eq "check") and -e $lock_on){
		my $mtime = time - (stat $lock_on)[9];
		while($mtime < 60){
			sleep 1;
			$mtime = time - (stat $lock_on)[9];
			last if(--$w_times < 0);
		}
		$success = ($mtime >= 60  ? rename $lock_on => $lock_off : 0);
		$success = (open(LOCK,">".$lock_on),close LOCK)		if(-e $lock_on and $success == 1);
	}else{
		$success = ($mode eq "on" ? rename $lock_off => $lock_on : rename $lock_on  => $lock_off);
		$success = (open(LOCK,">".$lock_on),close LOCK)		if(-e $lock_on and $success == 1);
	}
#		$n_time  = time;
#		$success = utime($n_time,$n_time,$lock_on)		if(-e $lock_on and $success == 1);
	return $success;
}

1;

__END__
